#include <GL/glut.h>
#include <stdio.h>

#define VIEWPORT_RATIO (4.0 / 3.0)
#define VIEWPORT_ASPECT 50.0

double rotate_x;
double rotate_y;

void draw_teapots()
{
    /* Teapot 1 */
    glTranslatef(-3, 0, 0);

    GLfloat material_ambient[] = {1, 0, 0, 1};
    GLfloat material_diffuse[] = {1, 0, 0, 1};
    GLfloat material_specular[] = {1, 1, 1, 1};

    glMaterialfv(GL_FRONT, GL_AMBIENT, material_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);

    GLfloat material_shininess[] = { 50.0 };
    glMaterialfv(GL_FRONT, GL_SHININESS, material_shininess);

    glutSolidTeapot(1.0); 

    /* Teapot 2 */
    glTranslatef(3, 0, 0);

    GLfloat material_ambient_2[] = {1, 1, 1, 1};
    GLfloat material_diffuse_2[] = {1, 1, 1, 1};
    glMaterialfv(GL_FRONT, GL_AMBIENT, material_ambient_2);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse_2);

    glutSolidTeapot(1.0); 

    /* Teapot 3 */
    glTranslatef(3, 0, 0);

    GLfloat material_ambient_3[] = {0, 1, 0, 1};
    GLfloat material_diffuse_3[] = {0, 1, 0, 1};
    glMaterialfv(GL_FRONT, GL_AMBIENT, material_ambient_3);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse_3);

    glutSolidTeapot(1.0); 
}

void set_lightings()
{
    GLfloat light_position[] = { -3.0, 0.0, 3.0, 0.0 };
    GLfloat light_ambient[] = { 0.1, 0.1, 0.1, 0 };
    GLfloat light_diffuse[] = { 0.5, 0.5, 0, 0 };
    GLfloat light_specular[] = { 1, 1, 1, 0 };

    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
}

void display()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();

    glRotatef(rotate_x, 1, 0, 0);
    glRotatef(rotate_y, 0, 1, 0);

    draw_teapots();

    glPopMatrix();

    glutSwapBuffers();
}

void reshape(GLsizei width, GLsizei height)
{
    int x, y, w, h;
    double ratio;

    ratio = (double)width / height;
    if (ratio > VIEWPORT_RATIO) {
        w = (int)((double)height * VIEWPORT_RATIO);
        h = height;
        x = (width - w) / 2;
        y = 0;
    }
    else {
        w = width;
        h = (int)((double)width / VIEWPORT_RATIO);
        x = 0;
        y = (height - h) / 2;
    }

    glViewport (x, y, w, h);
}

void initialize()
{
    glShadeModel(GL_SMOOTH);

    glEnable(GL_NORMALIZE);
    glEnable(GL_AUTO_NORMAL);

    glClearColor(0.0, 0.0, 0.0, 0.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(
        3.0, 3.0, 10.0, // eye
        0.0, 0.0, 0.0, // look at
        0.0, 0.0, 1.0  // up
    );

    glEnable(GL_DEPTH_TEST);

    set_lightings();

    glClearDepth(1.0);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(VIEWPORT_ASPECT, VIEWPORT_RATIO, 0.01, 1000.0);

    rotate_x = 0.0;
    rotate_y = 0.0;
}

void mouse_handler(int x, int y)
{
    rotate_x = x;
    rotate_y = y;
    glutPostRedisplay();
}

/**
 * Main function
 */
int main(int argc, char* argv[])
{
    glutInit(&argc, argv);
    glutInitWindowSize(640, 480);     
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);

    int window = glutCreateWindow("Square example");
    glutSetWindow(window);

    initialize();

    glutDisplayFunc(display);
    glutMotionFunc(mouse_handler);
    glutReshapeFunc(reshape);

    glutMainLoop();

    return 0;
}

